'use strict';
// Set to true to get noisier console.log statements
var VERBOSE_DEBUG = false;

if (window.location.origin + '/' === chrome.extension.getURL('')) {

  // above line avoids content scripts making their host page break frames
  if (window.top !== window)
    window.location.replace('about:blank');
}

// Global variable for Opera, so we can make specific things for Opera
var OPERA = navigator.userAgent.indexOf('OPR') > -1;

// Global const to hold thirty minutes in milliseconds
const THIRTY_MINUTES_IN_MILLISECONDS = 1800000;

// Run a function on the background page.
// Inputs (positional):
//   first, a string - the name of the function to call
//   then, any arguments to pass to the function (optional)
//   then, a callback:function(return_value:any) (optional)
var BGcall = function () {
  var args = [];
  for (var i = 0; i < arguments.length; i++)
    args.push(arguments[i]);
  var fn = args.shift();
  var has_callback = (typeof args[args.length - 1] == 'function');
  var callback = (has_callback ? args.pop() : function () {});

  chrome.extension.sendRequest({ command: 'call', fn: fn, args: args }, callback);
};

// Enabled in adblock_start_common.js and background.js if the user wants
var logging = function (enabled) {
  if (enabled) {
    loggingEnable = true;
    window.log = function () {
      if (VERBOSE_DEBUG || arguments[0] != '[DEBUG]') // comment out for verbosity
        console.log.apply(console, arguments);
    };
  } else {
    window.log = function () {
    };

    loggingEnable = false;
  }
};

logging(false); // disabled by default
var loggingEnable = false;

// Behaves very similarly to $.ready() but does not require jQuery.
var onReady = function (callback) {
    if (document.readyState === 'complete')
        window.setTimeout(callback, 0);
    else
        window.addEventListener('load', callback, false);
  };

// Precondition: initializeLangData must have been called
// and completed
var translate = function (messageID, args) {
  var message = "";
  if (Array.isArray(args)) {
    for (var i = 0; i < args.length; i++) {
      if (typeof args[i] !== 'string') {
        args[i] = args[i].toString();
      }
    }
  } else if (args && typeof args !== 'string') {
    args = args.toString();
  }

  // check if we should use dropdown language or chrome.i18n API
  if (useDropdownLang) {
    // user has set language using dropdown
    message = languageSelection.i18n.getMessage(messageID, args);
  } else {
    // use chrome.i18n API
    message = chrome.i18n.getMessage(messageID, args);
  }

  return message;
};

var splitMessageWithReplacementText = function(rawMessageText, messageID) {
    var anchorStartPos = rawMessageText.indexOf('[[');
    var anchorEndPos = rawMessageText.indexOf(']]');

    if (anchorStartPos === -1 || anchorEndPos === -1) {
      log("replacement tag not found", messageID, rawMessageText, anchorStartPos, anchorEndPos);
      return { error: "no brackets found" };
    }
    var returnObj = {};
    returnObj.anchorPrefixText = rawMessageText.substring(0, anchorStartPos);
    returnObj.anchorText = rawMessageText.substring(anchorStartPos + 2, anchorEndPos);
    returnObj.anchorPostfixText = rawMessageText.substring(anchorEndPos + 2);
    return returnObj;
};

// Wrapper function that finds the currently active language,
// then calls the inner _localizePage()
// Precondition: initializeLangData must have been called and
// completed
var localizePage = function () {
  var language = undefined;
  // find language to use, then complete localization
  if (useDropdownLang) {
    // user has set language using dropdown
    language = languageSelection.i18n.getActiveLocale();
  } else {
    // use chrome.i18n API
    language = determineUserLanguage();
  }
  _localizePage(language);
}

// Inner function that actually does the localization
var _localizePage = function (language) {

    //translate a page into the users language
    $('[i18n]:not(.i18n-replaced, [i18n_replacement_el])').each(function () {
        $(this).text(translate($(this).attr('i18n')));
    });

    $('[i18n_value]:not(.i18n-replaced)').each(function () {
        $(this).val(translate($(this).attr('i18n_value')));
    });

    $('[i18n_title]:not(.i18n-replaced)').each(function () {
        $(this).attr('title', translate($(this).attr('i18n_title')));
    });

    $('[i18n_placeholder]:not(.i18n-replaced)').each(function () {
        $(this).attr('placeholder', translate($(this).attr('i18n_placeholder')));
    });

  $("[i18n_replacement_el]:not(.i18n-replaced)").each(function() {
    // Replace a dummy <a/> inside of localized text with a real element.
    // Give the real element the same text as the dummy link.
    var messageID = $(this).attr("i18n");
    if (!messageID || typeof messageID !== "string") {
      $(this).addClass("i18n-replaced");
      return;
    }
    if (!$(this).get(0).firstChild) {
       log("returning, no first child found", $(this).attr("i18n"));
       return;
    }
    if (!$(this).get(0).lastChild) {
       log("returning, no last child found", $(this).attr("i18n"));
       return;
    }
    var replaceElId = '#' + $(this).attr("i18n_replacement_el");
    if ($(replaceElId).length === 0) {
      log("returning, no child element found", $(this).attr("i18n"), replaceElId);
      return;
    }
    var rawMessageText = translate(messageID) || "";
    var messageSplit = splitMessageWithReplacementText(rawMessageText, messageID);
    $(this).get(0).firstChild.nodeValue = messageSplit.anchorPrefixText;
    $(this).get(0).lastChild.nodeValue = messageSplit.anchorPostfixText;
    if (!$(this).hasClass("i18n-placeholder-replaced")) {
      if ($(replaceElId).get(0).tagName === "INPUT") {
        $('#' + $(this).attr("i18n_replacement_el")).prop('value', messageSplit.anchorText);
      } else {
        $('#' + $(this).attr("i18n_replacement_el")).text(messageSplit.anchorText);
      }
    }

    // If localizePage is run again, don't let the [i18n] code above
    // clobber our work
    $(this).addClass("i18n-replaced");
  });

    // Make a right-to-left translation for Arabic and Hebrew languages
    if (language === 'ar' || language === 'he') {
      $('#main_nav').removeClass('right').addClass('left');
      $('.adblock-logo').removeClass('left').addClass('right');
      $('.closelegend').css('float', 'left');
      document.documentElement.dir = 'rtl';
    }

  };

// Determine what language the user's browser is set to use
var determineUserLanguage = function () {
    if ((typeof navigator.language !== 'undefined') &&
        navigator.language)
        return navigator.language.match(/^[a-z]+/i)[0];
    else
        return null;
  };

// Parse a URL. Based upon http://blog.stevenlevithan.com/archives/parseuri
// parseUri 1.2.2, (c) Steven Levithan <stevenlevithan.com>, MIT License
// Inputs: url: the URL you want to parse
// Outputs: object containing all parts of |url| as attributes
var parseUri = function (url) {
    var matches = /^(([^:]+(?::|$))(?:(?:\w+:)?\/\/)?(?:[^:@\/]*(?::[^:@\/]*)?@)?(([^:\/?#]*)(?::(\d*))?))((?:[^?#\/]*\/)*[^?#]*)(\?[^#]*)?(\#.*)?/.exec(url);

    // The key values are identical to the JS location object values for that key
    var keys = ['href', 'origin', 'protocol', 'host', 'hostname', 'port',
        'pathname', 'search', 'hash',];
    var uri = {};
    for (var i = 0; (matches && i < keys.length); i++)
        uri[keys[i]] = matches[i] || '';
    return uri;
  };

// Parses the search part of a URL into an key: value object.
// e.g., ?hello=world&ext=adblock would become {hello:"world", ext:"adblock"}
// Inputs: search: the search query of a URL. Must have &-separated values.
parseUri.parseSearch = function (search) {

    // Fails if a key exists twice (e.g., ?a=foo&a=bar would return {a:"bar"}
    search = search.substring(search.indexOf('?') + 1).split('&');
    var params = {}, pair;
    for (var i = 0; i < search.length; i++) {
      pair = search[i].split('=');
      if (pair[0] && !pair[1])
          pair[1] = '';
      if (!params[decodeURIComponent(pair[0])] && decodeURIComponent(pair[1]) === 'undefined') {
        continue;
      } else {
        params[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1]);
      }
    }

    return params;
  };

// Strip third+ level domain names from the domain and return the result.
// Inputs: domain: the domain that should be parsed
//         keepDot: true if trailing dots should be preserved in the domain
// Returns: the parsed domain
parseUri.secondLevelDomainOnly = function (domain, keepDot) {
    var match = domain.match(/([^\.]+\.(?:co\.)?[^\.]+)\.?$/) || [domain, domain];
    return match[keepDot ? 0 : 1].toLowerCase();
  };

// Return |domain| encoded in Unicode
var getUnicodeDomain = function (domain) {
    if (domain) {
      return punycode.toUnicode(domain);
    } else {
      return domain;
    }
  };

// Return |url| encoded in Unicode
var getUnicodeUrl = function (url) {

    // URLs encoded in Punycode contain xn-- prefix
    if (url && url.indexOf('xn--') > 0) {
      var parsed = parseUri(url);
      // IDN domains have just hostnames encoded in punycode
      parsed.href = parsed.href.replace(parsed.hostname, punycode.toUnicode(parsed.hostname));
      return parsed.href;
    }

    return url;
  };

// Return obj[value], first setting it to |defaultValue| if it is undefined.
var setDefault = function (obj, value, defaultValue) {
    if (obj[value] === undefined)
        obj[value] = defaultValue;
    return obj[value];
  };

// Inputs: key:string.
// Returns value if key exists, else undefined.
var storage_get = function (key) {
  var store = localStorage;
  if (store === undefined) {
    return undefined;
  }

  var json = store.getItem(key);
  if (json == null)
    return undefined;
  try {
    return JSON.parse(json);
  } catch (e) {
    log("Couldn't parse json for " + key);
    return undefined;
  }
};

// Compatibility
var storageGet = storage_get;

// Inputs: key:string, value:object.
// If value === undefined, removes key from storage.
// Returns undefined.
var storage_set = function (key, value) {
  var store = localStorage;
  if (value === undefined) {
    store.removeItem(key);
    return;
  }

  try {
    store.setItem(key, JSON.stringify(value));
  } catch (ex) {
    // ToDo - handle errors gracefully.
  }
};

// Compatibility
var storageSet = storage_set;

var isEqual = function (x, y) {
  if ((typeof x == 'object' && x != null) && (typeof y == 'object' && y != null)) {
    if (Object.keys(x).length != Object.keys(y).length) {
      console.log('object keys length differ', Object.keys(x), Object.keys(y));
      return false;
    }

    for (var prop in x) {
      if (y.hasOwnProperty(prop)) {
        if (!isEqual(x[prop], y[prop])) {
          console.log('property values differ', prop, x[prop], y[prop]);
          return false;
        }
      } else {
        console.log("y doesn't have property ", prop);
        return false;
      }
    }

    return true;
  } else if (x !== y) {
    console.log("x and y don't = ", x, y);
    return false;
  } else {
    return true;
  }
};

// Sets expirable object in storage to be used in place of a cookie
// Inputs: 
//   name: string, 
//   value: object, 
//   millisecondsUntilExpire: number of milliseconds until the "cookie" expires
var setStorageCookie = function (name, value, millisecondsUntilExpire) {
  var expirationTime = Date.now() + (millisecondsUntilExpire || 0);

  storageSet(name, {value: value, expires: expirationTime});
}

// Returns value of storage "cookie" or undefined if the it doesn't exist or
// has expired
// Inputs: 
//  name: string
var getStorageCookie = function (name) {
  var storedCookie = storageGet(name);

  if (storedCookie && (storedCookie.expires > Date.now())) {
    return storedCookie.value;
  }

  return undefined;
}

// Asynchronously initializes language data if dropdown is set
// Must be called prior to any calls to localizePage
// Must be called prior to any calls to translate
// Returns a promise
var initializeLangData = function() {
  return new Promise(function(resolve) {
    if (typeof chrome.extension.getBackgroundPage !== 'function') {
      // running in content script
      if (useDropdownLang) {
        // user has set language using dropdown
        chrome.runtime.sendMessage("get_l10n_data", function(data) {
          languageSelection.i18n._setL10nData(data);
          resolve();
        });
      } else {
        resolve();
      }
    } else {
      // running in popup or options
      if (useDropdownLang) {
        // user has set language via dropdown
        var BG = chrome.extension.getBackgroundPage();
        BG.get_l10n_data().then(function(data) { 
          languageSelection.i18n._setL10nData(data); 
          resolve();
        });
      } else {
        resolve();
      }
    }
  });
}

// Initializes useDropdownLang variable
// Should only be called once in each context 
var initializeUseDropdownLang = function() {
  if (typeof chrome.extension.getBackgroundPage !== 'function') {
    // running in content script
    BGcall('getSelectedLocale', function(selectedLocale) {
      useDropdownLang = selectedLocale;
    });
  } else {
    // running in popup or options
    var BG = chrome.extension.getBackgroundPage();
    if (typeof BG.getSelectedLocale === 'function')
      useDropdownLang = BG.getSelectedLocale();
  }
}

// Global variable to keep track of language dropdown
// true if dropdown is enabled with a language selected
// false otherwise
var useDropdownLang = false;
initializeUseDropdownLang();

// Listen for a change to the language dropdown
chrome.runtime.onMessage.addListener(function(message) {
  if (message && message.name === "langDropdownChange") {
    useDropdownLang = message.data; 
  }
});

